@@ -1,7 +1,7 @@
 
 /*===========================================================================
    GNU UnRTF, a command-line program to convert RTF documents to other formats.
-   Copyright (C) 2000,2001 Zachary Thayer Smith
+   Copyright (C) 2000,2001,2004 Zachary Thayer Smith
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,15 +15,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-   The author is reachable by electronic mail at tuorfa@yahoo.com.
+   The maintainer is reachable by electronic mail at daved@physiol.usyd.edu.au
 ===========================================================================*/
 
 
 /*----------------------------------------------------------------------
  * Module name:    convert
- * Author name:    Zach Smith
+ * Author name:    Zachary Smith
  * Create date:    24 Jul 01
  * Purpose:        Performs conversion from RTF to other formats.
  *----------------------------------------------------------------------
@@ -45,12 +45,15 @@
  * 05 Aug 01, tuorfa@yahoo.com: added support for hyperlink data with \field
  * 06 Aug 01, tuorfa@yahoo.com: added support for several font attributes
  * 08 Aug 01, gommer@gmx.net: bugfix for picture storing mechanism
- * 08 Sep 01, tuorfa@yahoo.com: added use of PROGRAM_NAME
+ * 08 Sep 01, tuorfa@yahoo.com: added use of UnRTF
  * 11 Sep 01, tuorfa@yahoo.com: added support for JPEG and PNG pictures
  * 19 Sep 01, tuorfa@yahoo.com: added output personality support
  * 22 Sep 01, tuorfa@yahoo.com: added function-level comment blocks 
  * 23 Sep 01, tuorfa@yahoo.com: fixed translation of \'XX expressions
  * 08 Oct 03, daved@physiol.usyd.edu.au: more special character code
+ * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
+ * 29 Mar 05, daved@physiol.usyd.edu.au: more unicode support
+ * 31 Mar 05, daved@physiol.usyd.edu.au: strcat security bug fixed
  *--------------------------------------------------------------------*/
 
 #include <stdio.h>
@@ -70,6 +73,7 @@
 #include "attr.h"
 
 
+
 /*
 #define BINARY_ATTRS
 */
@@ -143,11 +147,14 @@ static char *hyperlink_base = NULL;
 
 void starting_body();
 void starting_text();
+#if 1 /* daved - 0.19.6 */
+void print_with_special_exprs (char *s);
+#endif
 
 
 static int banner_printed=FALSE;
 
-#if 1 /* daved - 0.19.0 */
+#if 0 /* daved - 0.19.0 removed in 0.19.5 */
 char *entity(int symbol);
 #endif
 
@@ -165,17 +172,9 @@ print_banner () {
 	if (!banner_printed) {
 		printf (op->comment_begin);
 	  	printf ("Translation from RTF performed by ");
-		printf ("%s, version ", PROGRAM_NAME);
+		printf ("UnRTF, version ");
 		printf ("%s", PROGRAM_VERSION);
 		printf (op->comment_end);
-
-		printf (op->comment_begin);
-	  	printf ("For information about this marvellous program,");
-		printf (op->comment_end);
-
-		printf (op->comment_begin);
-	  	printf ("please go to %s", PROGRAM_WEBSITE);
-		printf (op->comment_end);
 	}
 	banner_printed=TRUE;
 }
@@ -328,7 +327,7 @@ process_font_table (Word *w)
 
 	while(w) {
 		int num;
-		char name[255];
+		char name[BUFSIZ];
 		char *tmp;
 
 		if ((w2=w->child)) {
@@ -341,8 +340,15 @@ process_font_table (Word *w)
 				while(w2) {
 					tmp = word_string (w2);
 					if (tmp && tmp[0] != '\\')
-						strcat(name,tmp);
-
+					{
+						if(strlen(tmp) + strlen(name) > BUFSIZ - 1)
+						{
+							printf("invalid font table entry\n");
+							name[0] = 0;
+						}
+						else
+							strcat(name,tmp);
+					}
 					w2=w2->next;
 				}
 
@@ -469,7 +475,11 @@ process_info_group (Word *w)
 					while (w2) {
 						char *s2 = word_string(w2);
 						if (s2[0] != '\\') 
+#if 1 /* daved 0.19.6 */
+							print_with_special_exprs (s2);
+#else
 							printf ("%s", s2);
+#endif
 						w2=w2->next;
 					}
 					printf (op->document_title_end);
@@ -758,56 +768,74 @@ cmd_field (Word *w, int align, char has_param, short num) {
 
 			s = word_string(child);
 
-			if (!strcmp("\\*", s)) {
-				w2=child->next;
-				while (w2) {
-					char *s2 = word_string(w2);
-					if (s2 && !strcmp("\\fldinst", s2)) {
-						Word *w3;
-						w3=w2->next;
-#if 1 /* daved - 0.19.0 */
-						{
-						char *s;
-						char *s4;
-						Word *w4;
-						s = word_string(w3);
-						if(s && !strcmp(s, "SYMBOL") )
-						{
-						w4=w3->next;
-						while(w4 && !strcmp(word_string(w4), " "))
-							w4 = w4->next;
-						s4 = word_string(w4);
-						if(s4)
-							printf("%s", entity(atoi(s4)));
-						}
-						}
+			if (!strcmp("\\*", s))
+			{
+			    w2=child->next;
+			    while (w2)
+			    {
+				char *s2 = word_string(w2);
+				if (s2 && !strcmp("\\fldinst", s2))
+				{
+				    Word *w3;
+#if 1 /* daved - 0.19.2 */
+				    char *s;
+				    char *s4;
+				    Word *w4;
 #endif
-						while (w3 && !w3->child) {
-							w3=w3->next;
-						}
-						if (w3) w3=w3->child;
-						while (w3) {
-							char *s3=word_string(w3);
-							if (s3 && !strcmp("HYPERLINK",s3)) {
-								Word *w4;
-								char *s4;
-								w4=w3->next;
-								while (w4 && !strcmp(" ", word_string(w4)))
-									w4=w4->next;
-								if (w4) {
-									s4=word_string(w4);
-									printf (op->hyperlink_begin);
-									printf ("%s", s4);
-									printf (op->hyperlink_end);
-									return TRUE;
-								}
-								
-							}
-							w3=w3->next;
-						}
+				    w3=w2->next;
+#if 1 /* daved - 0.19.0 */
+				    s = word_string(w3);
+				    if(s && !strcmp(s, "SYMBOL") )
+				    {
+					w4=w3->next;
+					while(w4 && !strcmp(word_string(w4), " "))
+						w4 = w4->next;
+					s4 = word_string(w4);
+					if(s4)
+					{
+					   int char_num;
+					   char_num = atoi(s4);
+					   if
+					   (
+					        op->symbol_first_char <= char_num
+						&&
+						op->symbol_last_char >= char_num
+					   )
+					   {
+					   	char * string;
+						if((string = op->symbol_translation_table[char_num - op->symbol_first_char]) != 0)
+						printf("%s", string);
+					   }
 					}
-					w2=w2->next;
+				    }
+#endif
+				    while (w3 && !w3->child) {
+					    w3=w3->next;
+				    }
+				    if (w3) w3=w3->child;
+				    while (w3)
+				    {
+					    char *s3=word_string(w3);
+					    if (s3 && !strcmp("HYPERLINK",s3)) {
+						    Word *w4;
+						    char *s4;
+						    w4=w3->next;
+						    while (w4 && !strcmp(" ", word_string(w4)))
+							    w4=w4->next;
+						    if (w4) {
+							    s4=word_string(w4);
+							    printf (op->hyperlink_begin);
+							    printf ("%s", s4);
+							    printf (op->hyperlink_end);
+							    return TRUE;
+						    }
+					    	
+					    }
+					    w3=w3->next;
+				    }
 				}
+					w2=w2->next;
+			    }
 				
 			}
 		}
@@ -833,12 +861,21 @@ cmd_f (Word *w, int align, char has_param, short num) {
 		return FALSE;
 
 	name = lookup_fontname(num);
+#if 1 /* daved - 0.19.6 */
+	/* if a character is requested by number, we will need to know
+	   which translation table to use - roman or symbol */
+	numchar_table = FONTROMAN_TABLE;
+#endif
 	if(!name) {
 		printf (op->comment_begin);
 		printf ("invalid font number %d",num);
 		printf (op->comment_end);
 	} else {
 		attr_push(ATTR_FONTFACE,name);
+#if 1 /* daved - 0.19.6 */
+		if(strstr(name,"Symbol") != NULL)
+			numchar_table=FONTSYMBOL_TABLE;
+#endif
 	}
 
 	return FALSE;
@@ -1530,6 +1567,98 @@ static int cmd_up (Word *w, int align, char has_param, short param) {
 	return FALSE;
 }
 
+#if 1 /* daved - 0.19.4 */
+/*========================================================================
+ * Name:	cmd_u
+ * Purpose:	Processes a Unicode character
+ * Args:	Word, paragraph align info, and numeric param if any.
+ * Returns:	Flag, always false
+ *=======================================================================*/
+
+static int cmd_u (Word *w, int align, char has_param, short param) {
+	short	done=0;
+#if DEBUG
+	char	*str;
+	if(has_param == TRUE)
+	{
+		fprintf(stderr,"param is %u (%x)\n", (unsigned short)param,
+			(unsigned short)param);
+	}
+	if(w->hash_index)
+	{
+		str=hash_get_string (w->hash_index);
+		fprintf(stderr,"string is %s\n", str);
+	}
+#endif
+	if
+	(
+		param >= op->unisymbol1_first_char
+		&&
+		param <= op->unisymbol1_last_char
+	)
+	{
+		char *string;
+		if((string = op->unisymbol1_translation_table[param - op->unisymbol1_first_char]) != 0)
+			printf("%s", string);
+		else
+			printf("&#%u;", (unsigned short)param);
+		done++;
+	}
+	if
+	(
+		param >= op->unisymbol2_first_char
+		&&
+		param <= op->unisymbol2_last_char
+	)
+	{
+		char *string;
+		if((string = op->unisymbol2_translation_table[param - op->unisymbol2_first_char]) != 0)
+			printf("%s", string);
+		else
+			printf("&#%u;", (unsigned short)param);
+		done++;
+	}
+	if
+	(
+		param >= op->unisymbol3_first_char
+		&&
+		param <= op->unisymbol3_last_char
+	)
+	{
+		char *string;
+		if((string = op->unisymbol3_translation_table[param - op->unisymbol3_first_char]) != 0)
+			printf("%s", string);
+		else
+			printf("&#%u;", (unsigned short)param);
+		done++;
+	}
+#if 1 /* 0.19.5 more unicode support */
+	if
+	(
+		param >= op->unisymbol4_first_char
+		&&
+		param <= op->unisymbol4_last_char
+	)
+	{
+		char *string;
+		if((string = op->unisymbol4_translation_table[param - op->unisymbol4_first_char]) != 0)
+			printf("%s", string);
+		else
+			printf("&#%u;", (unsigned short)param);
+		done++;
+	}
+#endif
+	
+	/*
+	** if we know how to represent the unicode character in the
+	** output language, we need to skip the next word, otherwise
+	** we will output that alternative.
+	*/
+	if(done)
+		return(SKIP_ONE_WORD);
+	return(FALSE);
+}
+#endif
 
 /*========================================================================
  * Name:	cmd_dn
@@ -2285,6 +2414,9 @@ static HashItem hashArray_t [] = {
 	{ NULL, NULL, NULL}
 };
 static HashItem hashArray_u [] = {
+#if 1 /* daved - 0.19.4 */
+	{ "u", &cmd_u, NULL },
+#endif
 	{ "ul", &cmd_ul, NULL },
 	{ "up", &cmd_up, NULL },
 	{ "uld", &cmd_uld, NULL },
@@ -2369,6 +2501,7 @@ enum { SMALL=0, BIG=1 };
 
 	CHECK_PARAM_NOT_NULL(s);
 
+	state=SMALL;  /* Pacify gcc,  st001906 - 0.19.6 */
 	if (simulate_smallcaps) {
 		if (*s >= 'a' && *s <= 'z') {
 			state=SMALL;
@@ -2385,7 +2518,11 @@ enum { SMALL=0, BIG=1 };
 			ch = toupper (ch);
 
 		if (ch >= 0x20 && ch < 0x80) {
+#if 1 /* daved - 0.19.6 */
+			post_trans = op_translate_char (op, charset_type, ch, numchar_table);
+#else
 			post_trans = op_translate_char (op, charset_type, ch);
+#endif
 			printf ("%s",post_trans);
 		}
 
@@ -2656,9 +2793,29 @@ word_print_core (Word *w)
 						printf (op->word_end);
 				}
 
-			/*---Handle RTF keywords---------------------------*/
 
-			} else {
+			}
+#if 1 /* daved 0.19.5 */
+			/* output an escaped backslash */
+			/* do we need special handling for latex? */
+			else if(*(s+1) == '\\')
+			{
+				s++;
+				putchar('\\');
+			}
+			else if(*(s+1) == '{')
+			{
+				s++;
+				putchar('{');
+			}
+			else if(*(s+1) == '}')
+			{
+				s++;
+				putchar('}');
+			}
+#endif
+			/*---Handle RTF keywords---------------------------*/
+			else {
 				int done=FALSE;
 
 				s++;
@@ -2721,7 +2878,11 @@ word_print_core (Word *w)
 					int ch = h2toi (&s[1]);
 					char *s2;
 
+#if 1 /* daved - 0.19.6 */
+					s2 = op_translate_char (op, charset_type, ch, numchar_table);
+#else
 					s2 = op_translate_char (op, charset_type, ch);
+#endif
 
 					if (!s2 || !*s2) {
 						printf (op->comment_begin);
@@ -2743,7 +2904,7 @@ word_print_core (Word *w)
 					int have_param=FALSE, param=0;
 					HashItem *hip;
 					char *p;
-					int match;
+					int match=FALSE;  /* Pacify gcc,  st001906 - 0.19.6 */
 
 					/* Look for a parameter */
 					p=s;
@@ -2791,6 +2952,11 @@ word_print_core (Word *w)
 								if (hip[index].func) {
 									terminate_group = hip[index].func (w,paragraph_align, have_param, param);
 
+#if 1 /* daved - 0.19.4 - unicode support may need to skip only one word */
+									if(terminate_group == SKIP_ONE_WORD)
+											w=w->next;
+											else
+#endif
 									if (terminate_group)
 										while(w) w=w->next;
 								}
