@@ -1,6 +1,6 @@
 /*=============================================================================
    GNU UnRTF, a command-line program to convert RTF documents to other formats.
-   Copyright (C) 2000,2001 Zachary Thayer Smith
+   Copyright (C) 2000,2001,2004 by Zachary Smith
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,15 +14,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
-   The author is reachable by electronic mail at tuorfa@yahoo.com.
+   The maintainer is reachable by electronic mail at daved@physiol.usyd.edu.au
 =============================================================================*/
 
 
 /*----------------------------------------------------------------------
  * Module name:    main.c
- * Author name:    Zach Smith
+ * Author name:    Zachary Smith
  * Create date:    01 Sep 00
  * Purpose:        main() routine with file open/close.
  *----------------------------------------------------------------------
@@ -32,20 +32,23 @@
  * 08 Apr 01, tuorfa@yahoo.com: more GNU-like switches implemented
  * 24 Jul 01, tuorfa@yahoo.com: removed verify_file_type()
  * 03 Aug 01, tuorfa@yahoo.com: added --inline switch
- * 08 Sep 01, tuorfa@yahoo.com: added use of PROGRAM_NAME
+ * 08 Sep 01, tuorfa@yahoo.com: added use of UnRTF
  * 19 Sep 01, tuorfa@yahoo.com: addition of output personalities
  * 22 Sep 01, tuorfa@yahoo.com: added function-level comment blocks 
  * 23 Sep 01, tuorfa@yahoo.com: added wpml switch
  * 08 Oct 03, daved@physiol.usyd.edu.au: added stdlib.h for linux
+ * 07 Jan 04, tuorfa@yahoo.com: removed broken PS support
+ * 25 Sep 04, st001906@hrz1.hrz.tu-darmstadt.de: added stdlib.h for djgpp
+ * 29 Mar 05, daved@physiol.usyd.edu.au: changes requested by ZT Smith
  *--------------------------------------------------------------------*/
 
 
 
 #include <stdio.h>
-#include <string.h>
-#if linux /* daved - 0.19.0 */
+#if linux || __DJGPP__ /* daved - 0.19.0, st001906 - 0.19.6 */
 #include <stdlib.h>
 #endif
+#include <string.h>
 
 #include "defs.h"
 #include "error.h"
@@ -59,9 +62,7 @@
 #include "html.h"
 #include "text.h"
 #include "vt.h"
-#include "ps.h"
 #include "latex.h"
-#include "wpml.h"
 
 
 
@@ -108,9 +109,7 @@ main (int argc, char **argv)
 		else if (!strcmp("--html",argv[i])) output_format=OP_HTML;
 		else if (!strcmp("--text",argv[i])) output_format=OP_TEXT;
 		else if (!strcmp("--vt",argv[i])) output_format=OP_VT;
-		else if (!strcmp("--ps",argv[i])) output_format=OP_PS;
 		else if (!strcmp("--latex",argv[i])) output_format=OP_LATEX;
-		else if (!strcmp("--wpml",argv[i])) output_format=OP_WPML;
 		else if (!strcmp("-t",argv[i])) {
 			if ((i+1)<argc && *argv[i+1]!='-') {
 				i++;
@@ -120,12 +119,8 @@ main (int argc, char **argv)
 					output_format=OP_VT;
 				else if (!strcmp ("text", argv[i]))
 					output_format=OP_TEXT;
-				else if (!strcmp ("ps", argv[i]))
-					output_format=OP_PS;
 				else if (!strcmp ("latex", argv[i]))
 					output_format=OP_LATEX;
-				else if (!strcmp ("wpml", argv[i]))
-					output_format=OP_WPML;
 			}
 		} 
 		else if (!strcmp("--inline",argv[i])) inline_mode=TRUE;
@@ -161,25 +156,19 @@ main (int argc, char **argv)
 	case OP_HTML:
 		op = html_init();
 		break;
-	case OP_PS:
-		op = ps_init();
-		break;
 	case OP_LATEX:
 		op = latex_init();
 		break;
-	case OP_WPML:
-		op = wpml_init();
-		break;
 	default:
 		error_handler ("unknown output format");
 	}
 
 	hash_init();
 
-	fprintf (stderr, "This is %s, ", PROGRAM_NAME);
+	fprintf (stderr, "This is UnRTF ");
 	fprintf (stderr, "version %s\n", PROGRAM_VERSION);
 	fprintf (stderr, "By Dave Davey and Marcos Serrou do Amaral\n");
-	fprintf (stderr, "Original Author: Zach T. Smith\n");
+	fprintf (stderr, "Original Author: Zachary Smith\n");
 
 	if (debug_mode) fprintf (stderr, "Debug mode.\n");
 	if (dump_mode) fprintf (stderr, "Dump mode.\n");
